searchData={"items":[{"type":"module","title":"Taido","doc":"`Taido` is a library for building and executing behavior trees in Elixir.\n\nFirst, build your tree using functions from the `Taido.Node` module.\nHere's an example of a behavior tree for a spaceship undocking from a\nspace station and traveling to another waypoint.\n\n    alias Taido.Node\n\n    # At its highest level, we have two steps: undock, and navigate.\n    # A sequence executes each of its children in order until\n    # one fails.\n    Node.sequence([\n\n      # We want to undock, but it is possible that we are already undocked.\n      # A `select` node, also known as a fallback, executes its children\n      # in order until one of them succeeds.\n      Node.select([\n\n        # If you want to make your behavior tree more modular, decorator\n        # nodes like `invert` allow you to flip the result of another node.\n        # Here, we will use it to flip a `docked?` condition, so that\n        # this step will succeed if we are undocked.\n        # If it succeeds, this node's parent will not execute the subsequent\n        # children, because it is a select node.\n        Node.invert(\n\n          # A condition node takes a boolean function, and succeeds if it\n          # returns `true`, else it fails.\n          # This node will succeed if the ship is docked.\n          Node.condition(fn state ->\n            state.ship.docked?\n          end)\n        ),\n\n        # Finally, we do something to affect the world. The `action` node\n        # gives you the tree's state object, and lets you modify it, then\n        # lets you return a success or failure status of your own.\n        # This will actually undock our ship, but only if the preceding\n        # child didn't succeed, because they're both inside a `select` node.\n        Node.action(fn state ->\n          {:ok, ship} = Spaceships.undock(state.ship)\n\n          {:success, Map.put(state, :ship, ship)}\n        end)\n\n        # Notice that our `select` node ends here.\n      ]),\n\n      # The goal of this entire tree, we navigate. Since this node is\n      # a child of a `sequence` node, it gets executed only if its preceding\n      # sibling nodes all succeed.\n      Node.action(fn state ->\n        {:ok, ship} = Spaceships.navigate(state.ship, \"MARS\")\n\n        {:success, Map.put(state, :ship, ship)}\n      end)\n    ])\n\nThen you can execute the tree with `Taido.tick/2`.\nThis will evaluate every node in order until either the tree was fully\nevaluated, or until one of the nodes returned a `:running` status.\n\n    state = %{ship: Spaceships.new()}\n\n    {status, updated_tree, updated_state} = Taido.tick(tree, state)\n\nTwo important concepts in `Taido` are _state_ and _status_.\n- Taido treats state similarly to a `GenServer`'s state.\n  Every time a behavior tree is evaluated, you must provide a `state`\n  variable. This can be whatever data type you want; Taido only passes it\n  into each node in the tree so that you can update it, and returns the\n  updated state to you.\n- A node returns a status of either `:success`, `:failure`, or `:running`.\n  They _can_ mean the same thing as Elixir's `:ok` or `:error` tuples,\n  however they are mainly just feedback that a node gives to its parent,\n  and eventually to you.\n\nSee `Taido.Node` for the nodes provided by this library.\nYou can also create your own node by implementing the `Taido.BehaviorTree`\nprotocol.\n\nTaido's behavior tree nodes are structs, and the `Taido.tick/2` function\nis pure and runs in a single process, so the only side-effects are the ones\nyou bring with you.","ref":"Taido.html"},{"type":"module","title":"Asynchronous nodes - Taido","doc":"Asynchronous nodes are available, like `Taido.Node.async_action/1`.\nThat node runs your action in a `Task` and immediately returns `:running`,\nwhich causes the behavior tree to stop evaluating, saving its place to\nresume later. Every time you run `Taido.tick/2`, the task is checked,\nand if it is done, its result is fetched, and the tree continues evaluating\nlike normal.\n\n> #### Warning {: .warning}\n>\n> If you are running a behavior tree inside of anything like a `GenServer`,\n> which automatically await tasks, you must forward messages sent to the\n> `GenServer` to `Taido.handle_message/2`. Otherwise, the tasks in your\n> behavior tree will never be completed. It's as simple as this:\n>\n>     def handle_info(msg, state) do\n>       Taido.handle_message(state.behavior_tree, msg)\n>     end","ref":"Taido.html#module-asynchronous-nodes"},{"type":"function","title":"Taido.handle_message/2","doc":"Handle a process message.\n\nYou will only need to use this function if you are ticking asynchronous\nnodes in a context that automatically awaits tasks, like inside\na `GenServer`. In that case, you must forward the `GenServer`'s messages\nto `Taido` like this:\n\n    def handle_info(msg, state) do\n      Taido.handle_message(state.behavior_tree, msg)\n    end","ref":"Taido.html#handle_message/2"},{"type":"function","title":"Taido.terminate/1","doc":"Terminates all asynchronous tasks in the behavior tree.","ref":"Taido.html#terminate/1"},{"type":"function","title":"Taido.tick/2","doc":"Evaluate the behavior tree.\n\nReturns a tuple of `{status, updated_tree, your_state}`.\nMake sure to save the `updated_tree` somewhere, especially if you are using\nasynchronous nodes, because you must keep track of which tasks are\ncurrently waiting to be checked.","ref":"Taido.html#tick/2"},{"type":"protocol","title":"Taido.BehaviorTree","doc":"","ref":"Taido.BehaviorTree.html"},{"type":"function","title":"Taido.BehaviorTree.handle_message/2","doc":"","ref":"Taido.BehaviorTree.html#handle_message/2"},{"type":"function","title":"Taido.BehaviorTree.terminate/1","doc":"","ref":"Taido.BehaviorTree.html#terminate/1"},{"type":"function","title":"Taido.BehaviorTree.tick/2","doc":"","ref":"Taido.BehaviorTree.html#tick/2"},{"type":"type","title":"Taido.BehaviorTree.t/0","doc":"All the types that implement this protocol.","ref":"Taido.BehaviorTree.html#t:t/0"},{"type":"module","title":"Taido.Node","doc":"Nodes for `Taido`'s behavior trees.\n\nLeaf nodes:\n- `condition/1`\n- `action/1`\n\nAsynchronous leaf nodes:\n- `async_action/1`\n\nComposite nodes:\n- `select/1`\n- `sequence/1`\n\nDecorators:\n- `invert/1`","ref":"Taido.Node.html"},{"type":"function","title":"Taido.Node.action/1","doc":"","ref":"Taido.Node.html#action/1"},{"type":"function","title":"Taido.Node.async_action/1","doc":"","ref":"Taido.Node.html#async_action/1"},{"type":"function","title":"Taido.Node.condition/1","doc":"","ref":"Taido.Node.html#condition/1"},{"type":"function","title":"Taido.Node.invert/1","doc":"","ref":"Taido.Node.html#invert/1"},{"type":"function","title":"Taido.Node.select/1","doc":"","ref":"Taido.Node.html#select/1"},{"type":"function","title":"Taido.Node.sequence/1","doc":"","ref":"Taido.Node.html#sequence/1"},{"type":"module","title":"Taido.Node.Action","doc":"","ref":"Taido.Node.Action.html"},{"type":"module","title":"Taido.Node.AsyncAction","doc":"","ref":"Taido.Node.AsyncAction.html"},{"type":"module","title":"Taido.Node.Condition","doc":"","ref":"Taido.Node.Condition.html"},{"type":"module","title":"Taido.Node.Inverter","doc":"","ref":"Taido.Node.Inverter.html"},{"type":"module","title":"Taido.Node.Select","doc":"A composite node of behavior trees that takes a series of actions until\none succeeds, at which point this node is considered successful.","ref":"Taido.Node.Select.html"},{"type":"module","title":"Taido.Node.Sequence","doc":"A composite node of behavior trees that takes a series of actions until\none fails, at which point this node is considered failed.","ref":"Taido.Node.Sequence.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}